package com.good.domain;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonValue;

/**
 * Uses as response of create/re-new certificate. Object will be serialized into JSON and will be sent to the client.
 *
 * @author Stanislav Kyfenko 29.07.2015
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class P12CertificateData {
    /**
     * PKCS12 keystore. Mandatory field
     */
	private byte[] payload;
    /**
     * type of keystore. Mandatory field.
     */
	private CertificateTypeEnum payloadType;
    /**
     * Status of operation(success/failure). Mandatory field.
     */
	private StatusTypeEnum status;
    /**
     * information about failure. Should be presented only when {@link P12CertificateData#status} equals 
	 * {@link com.good.domain.P12CertificateData.StatusTypeEnum#FAILURE}. List of available errors can be checked in
	 * {@link P12CertificateFailure}.
     */
    private P12CertificateFailure failureInfo;
    /**
     * request ID should be same as field in {@link GDP12CertificateRequest#reqId}. Is needed for client to separate
	 * different responses.
     */
	private String reqId;
    /**
     * password for keystore, is not mandatory field. Should be presented only when 
	 * {@link GDP12CertificateRequest#authToken} was empty and server created PKCS12 keystore with randomly generated 
	 * password.
     */
    private String password;

	public P12CertificateData(P12CertificateFailure failureInfo) {
		this(failureInfo, null);
	}

	public P12CertificateData(byte[] payload,
			CertificateTypeEnum payloadType) {
		this.payload = payload;
		this.payloadType = payloadType;
		this.status = StatusTypeEnum.SUCCESS;
	}

	public P12CertificateData(P12CertificateFailure failureInfo, String reqId) {
		this.status = StatusTypeEnum.FAILURE;
		this.failureInfo = failureInfo;
		this.reqId = reqId;
	}

	public byte[] getPayload() {
		return this.payload;
	}

	public CertificateTypeEnum getPayloadType() {
		return this.payloadType;
	}

	public StatusTypeEnum getStatus() {
		return this.status;
	}

	public String getPassword() {
		return password;
	}

	public String getReqId() {
		return reqId;
	}

	public void setReqId(String reqId) {
		this.reqId = reqId;
	}

    public void setPassword(String password) {
        this.password = password;
    }

	public P12CertificateFailure getFailureInfo() { return this.failureInfo;	}

    @Override
    public String toString() {
        return "P12CertificateData{" +
                "payload.length=" + (payload == null ? null : payload.length) +
                ", payloadType=" + payloadType +
                ", status=" + status +
                ", failureInfo=" + failureInfo +
                ", reqId='" + reqId + '\'' +
                '}';
    }

    public enum CertificateTypeEnum {

		PKCS12("pkcs12");
		private final String val;

		CertificateTypeEnum(String val) {
			this.val = val;
		}

		@JsonValue
		public String getValue() {
			return this.val;
		}

		public static CertificateTypeEnum getByValue(String value) {
			switch (value) {
			case "pkcs12":
				return PKCS12;
			default:
				return null;
			}
		}

	}

	public enum StatusTypeEnum {

		SUCCESS("success"), FAILURE("failure");
		private final String val;

		StatusTypeEnum(String val) {
			this.val = val;
		}

		@JsonValue
		public String getValue() {
			return this.val;
		}

		public static StatusTypeEnum getByValue(String value) {
			switch (value) {
			case "success":
				return SUCCESS;
			case "failure":
				return FAILURE;
			default:
				return null;
			}
		}
	}
}
